@@ -481,6 +481,7 @@ static int send_wontcmd(const char *name, const char *) {
 int send_tncmd(int (*func)(int, int), const char *cmd, const char *name) {
     char **cpp;
     extern char *telopts[];
+    long opt;
 
     if (isprefix(name, "help") || isprefix(name, "?")) {
 	register int col, len;
@@ -507,16 +508,23 @@ int send_tncmd(int (*func)(int, int), const char *cmd, const char *name) {
 					name, cmd);
 	return 0;
     }
+
+    opt = cpp - telopts;
     if (cpp == 0) {
-	fprintf(stderr, "'%s': unknown argument ('send %s ?' for help).\n",
+	char *end;
+
+	opt = strtol(name, &end, 10);
+	if (*end || opt < 0 || opt > 255) {
+	    fprintf(stderr, "'%s': unknown argument ('send %s ?' for help).\n",
 					name, cmd);
-	return 0;
+	    return 0;
+	}
     }
     if (!connected) {
 	printf("?Need to be connected first.\n");
 	return 0;
     }
-    (*func)(cpp - telopts, 1);
+    (*func)(opt, 1);
     return 1;
 }
 
@@ -1701,7 +1709,7 @@ int tn(int argc, const char *argv[]) {
     char *srp = NULL;
     int srlen;
     int family = 0;
-    const char *cmd, *volatile user = 0;
+    const char *cmd, *volatile user = 0, *srchostp = 0;
     const char *portp = NULL;
     char *hostp = NULL;
     char *resolv_hostp;
@@ -1748,6 +1756,14 @@ int tn(int argc, const char *argv[]) {
 	    --argc;
 	    continue;
 	}
+	if (strcmp(*argv, "-b") == 0) {
+	    --argc; ++argv;
+	    if (argc == 0)
+		goto usage;
+	    srchostp = *argv++;
+	    --argc;
+	    continue;
+	}
 	if (strcmp(*argv, "-a") == 0) {
 	    --argc; ++argv;
 	    autologin = 1;
@@ -1830,6 +1846,20 @@ int tn(int argc, const char *argv[]) {
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_flags = AI_NUMERICHOST;
     hints.ai_family = family;
+
+    if (srchostp) {
+	res = getaddrinfo(srchostp, "0", &hints, &hostaddr);
+	if (res) {
+	    fprintf(stderr, "telnet: could not resolve %s: %s\n", srchostp,
+		    gai_strerror(res));
+	    return 0;
+	}
+	hints.ai_family = hostaddr->ai_family;
+	res = nlink.bind(hostaddr);
+	freeaddrinfo(hostaddr);
+	if (res < 0)
+	    return 0;
+    }
         
     /* Resolve both the host and service simultaneously. */
     res = getaddrinfo(resolv_hostp, portp, &hints, &hostaddr);
@@ -1883,16 +1913,16 @@ nextaddr:
 	auth_encrypt_connect(connected);
 #endif
     } while (connected == 0);
-    if (hostaddr->ai_canonname == 0) {
+    if (tmpaddr->ai_canonname == 0) {
 	hostname = new char[strlen(hostp)+1];
 	strcpy(hostname, hostp);
     }
     else {
-	hostname = new char[strlen(hostaddr->ai_canonname)+1];
-	strcpy(hostname, hostaddr->ai_canonname);
+	hostname = new char[strlen(tmpaddr->ai_canonname)+1];
+	strcpy(hostname, tmpaddr->ai_canonname);
     }
 
-    cmdrc(hostp, hostname);
+    cmdrc(hostp, hostname, portp);
     freeaddrinfo(hostaddr);
     if (autologin && user == NULL) {
 	struct passwd *pw;
@@ -2144,15 +2174,21 @@ static int help(command_table *tab, int argc, const char *argv[]) {
     return 0;
 }
 
-static void readrc(const char *m1, const char *m2, const char *rcname) {
+static void readrc(const char *m1, const char *m2, const char *port,
+		   const char *rcname)
+{
     FILE *rcfile;
     int gotmachine = 0;
     int l1 = strlen(m1);
     int l2 = strlen(m2);
-    char m1save[strlen(m1) + 1];
+    int lport = strlen(port);
+    char m1save[l1 + 1];
+    char portsave[lport + 1];
 
     strcpy(m1save, m1);
     m1 = m1save;
+    strcpy(portsave, port);
+    port = portsave;
 
     rcfile = fopen(rcname, "r");
     if (!rcfile) return;
@@ -2177,6 +2213,13 @@ static void readrc(const char *m1, const char *m2, const char *rcname) {
 		strncpy(line, &line[7], sizeof(line) - 7);
 	    else
 		continue;
+
+	    if (line[0] == ':') {
+		if (!strncasecmp(&line[1], port, lport))
+		    continue;
+		strncpy(line, &line[lport + 1], sizeof(line) - lport - 1);
+	    }
+
 	    if (line[0] != ' ' && line[0] != '\t' && line[0] != '\n')
 		continue;
 	    gotmachine = 1;
@@ -2189,13 +2232,13 @@ static void readrc(const char *m1, const char *m2, const char *rcname) {
     fclose(rcfile);
 }
 
-void cmdrc(const char *m1, const char *m2) {
+void cmdrc(const char *m1, const char *m2, const char *port) {
     static char *rcname = 0;
     static char rcbuf[128];
 
     if (skiprc) return;
 
-    readrc(m1, m2, "/etc/telnetrc");
+    readrc(m1, m2, port, "/etc/telnetrc");
     if (rcname == 0) {
 	rcname = getenv("HOME");
 	if (rcname)
@@ -2205,7 +2248,7 @@ void cmdrc(const char *m1, const char *m2) {
 	strcat(rcbuf, "/.telnetrc");
 	rcname = rcbuf;
     }
-    readrc(m1, m2, rcname);
+    readrc(m1, m2, port, rcname);
 }
 
 #if defined(IP_OPTIONS) && defined(HAS_IPPROTO_IP)
