@@ -96,20 +96,56 @@ void netlink::close(int doshutdown) {
     }
 #endif /* USE_SSL */
     ::close(net);
+    net = -1;
+}
+
+int netlink::bind(struct addrinfo *addr)
+{
+    int res;
+
+    res = socket(addr->ai_family);
+    if (res < 2) {
+	if (res == 1)
+	    perror("telnet: socket");
+	return -1;
+    }
+
+    if (::bind(net, addr->ai_addr, addr->ai_addrlen) < 0) {
+	perror("telnet: bind");
+	return -1;
+    }
+
+    return 0;
+}
+
+int netlink::socket(int family)
+{
+    if (this->family != family)
+	close(0);
+
+    if (net < 0) {
+	this->family = family;
+	net = ::socket(family, SOCK_STREAM, 0);
+	if (net < 0) {
+	    if (errno == EAFNOSUPPORT)
+		return 1;
+	    perror("telnet: socket");
+	    return 0;
+	}
+    }
+
+    return 2;
 }
 
 int netlink::connect(int debug, struct addrinfo *addr, 
 		     char *srcroute, int srlen, int tos) 
 {
     int on=1;
+    int res;
 
-    net = socket(addr->ai_family, SOCK_STREAM, 0);
-    if (net < 0) {
-	if (errno == EAFNOSUPPORT)
-	    return 1;
-	perror("telnet: socket");
-	return 0;
-    }
+    res = socket(addr->ai_family);
+    if (res < 2)
+	return res;
 
 #if defined(IP_OPTIONS) && defined(HAS_IPPROTO_IP)
     if (srcroute) {
