@@ -1,11 +1,15 @@
 
 class netlink {
+ private:
+    int family;
  protected:
     int net;
  public:
     netlink();
     ~netlink();
 
+    int bind(struct addrinfo *hostaddr);
+    int socket(int family);
     int connect(int debug, struct addrinfo *hostaddr, 
 		char *srcroute, int srlen,
 		int tos);
