@@ -10,12 +10,10 @@ void TerminalDefaultChars(void);
 int TerminalSpecialChars(int);
 void TerminalSpeeds(long *ispeed, long *ospeed);
 int TerminalWindowSize(long *rows, long *cols);
-#if 0
 void auth_encrypt_user(char *);
 void auth_name(unsigned char *, int);
 void auth_printsub(unsigned char *, int, unsigned char *, int);
-#endif
-void cmdrc(const char *m1, const char *m2);
+void cmdrc(const char *, const char *, const char *);
 void env_init(void);
 int getconnmode(void);
 void init_network(void);
